#pragma once

#include <memory>
#include <string>
#include <vector>
#include <unordered_map>

#include "utils/Random.hpp"

struct Texture;
struct RenderTexture;
struct Color;
struct Vector2;
struct Rectangle;

namespace panda
{
  struct TextureAtlasItem
  {
    int x;
    int y;
    int w;
    int h;
    float px;
    float py;
  };

  class Scene {
  public:
    Scene() {}
    virtual ~Scene() {}
    virtual void update() {}
  };

  struct Sprite {
    Texture *texture;
    float x = 0.0f;
    float y = 0.0f;
    float scaleX = 1.0f;
    float scaleY = 1.0f;
    float rotation = 0.0f;

    void Load(const char* path);
    void Unload();
    void Draw(Color color);

    void SetPivot(Vector2 ratio);
    void SetPivotRatio(Vector2 ratio);
    void SetFlip(bool x, bool y);
    void SetFlipX(bool x);
    void SetFlipY(bool y);

    Vector2 GetPivot();
    Vector2 GetPivotFlip();
    bool GetFlipX();
    bool GetFlipY();

    private:
      bool flipX = false;
      bool flipY = false;
      float pivotX = 0.0f;
      float pivotY = 0.0f;
      float pivotX_flip = 0.0f;
      float pivotY_flip = 0.0f;

      void CalculatePivotFlip();
  };

  struct Matrix {
    float a = 1.0f;
    float b = 0.0f;
    float c = 0.0f;
    float d = 1.0f;
    float tx = 0.0f;
    float ty = 0.0f;

    void identity();
    void translate(float dx, float dy);
    void rotate(float radian);
    void scale(float sx, float sy);

    void concat(Matrix m);
  };

  // ############################################################################################

  // Core
  void CreateWindow(int w, int h, const char *str);

  void AppUpdate();
  void SetScene(Scene* scene);
  Scene* GetScene();

  // # Graphic

  // ### GUI
  void GuiInit();
  void GuiDestroy();
  void GuiBegin();
  void GuiEnd();

  // ### atlas
  bool LoadTextureAtlas(const char *image_path, const char *ini_path, std::string AtlasKey);
  bool UnloadTextureAtlas(std::string AtlasKey);
  void DrawTextureAtlas(std::string AtlasKey, std::string key, int x, int y);
  void DrawTextureAtlasPro(std::string AtlasKey, std::string key, int x, int y, Color color, bool flipX = false, bool flipY = false);

  // ### draw
  void DrawTextureTiled(Texture texture, Rectangle source, Rectangle dest, Vector2 origin, float rotation, float scale, Color tint);
  void DrawRenderTexture(const RenderTexture &buffer, const float &x, const float &y, const Color &color);
  
  // ### texture utils
  Rectangle GetTextureRect(const Texture &tex, bool flipX, bool flipY);

  // Utils
  Random &GetDefaultRandom();

  // Euclidean Distance
  double distance(const Vector2 &p1, const Vector2 &p2);
  double distanceSqrt(const Vector2 &p1, const Vector2 &p2);

  // Manhattan Distance
  double distanceManhattan(const Vector2 &p1, const Vector2 &p2);
}
