#include <raylib.h>

#include "Panda.hpp"


namespace panda {
  void Sprite::Load(const char* path)
  {
    texture = new Texture(LoadTexture(path));
  }

  void Sprite::Unload()
  {
    UnloadTexture(*texture);
  }

  void Sprite::Draw(Color color)
  {
    Vector2 pivot_temp = (flipX || flipY) ? Vector2{pivotX_flip, pivotY_flip} : Vector2{pivotX, pivotY};

    panda::Matrix matrix;
    matrix.translate(-pivot_temp.x, -pivot_temp.y);
    matrix.scale(scaleX, scaleY);
    matrix.rotate(rotation * DEG2RAD);
    matrix.translate(x, y);

    DrawTexturePro(
      *texture,
      panda::GetTextureRect(*texture, flipX, flipY),
      {
        matrix.tx, matrix.ty,
        scaleX*texture->width,
        scaleY*texture->height
      },
      {0.0f, 0.0f},
      rotation, color
    );
  }

  void Sprite::SetPivot(Vector2 p)
  {
    pivotX = p.x;
    pivotY = p.y;
    CalculatePivotFlip();
  }

  void Sprite::SetPivotRatio(Vector2 ratio)
  {
    pivotX = texture->width*ratio.x;
    pivotY = texture->height*ratio.y;
    CalculatePivotFlip();
  }

  void Sprite::SetFlip(bool x, bool y)
  {
    flipX = x;
    flipY = y;
    CalculatePivotFlip();
  }

  void Sprite::SetFlipX(bool x)
  {
    flipX = x;
    CalculatePivotFlip();
  }

  void Sprite::SetFlipY(bool y)
  {
    flipY = y;
    CalculatePivotFlip();
  }

  Vector2 Sprite::GetPivot()
  {
    return {pivotX, pivotY};
  }

  Vector2 Sprite::GetPivotFlip()
  {
    return {pivotX_flip, pivotY_flip};
  }

  bool Sprite::GetFlipX()
  {
    return flipX;
  }

  bool Sprite::GetFlipY()
  {
    return flipY;
  }

  void Sprite::CalculatePivotFlip()
  {
    pivotX_flip = flipX ? texture->width-pivotX : pivotX;
    pivotY_flip = flipY ? texture->height-pivotY : pivotY;
  }
}